/*             _   
    /| /| /|  / \      (C) 2001  Martin Suchanek
     |/ |/  \_\_/          xsuchan@fi.muni.cz
                \   Student at Faculty of Informatics
                |       Masaryk University Brno
            \__/
*/

#include "Board.h"


/*----------------------------------------------------
	Function: Board
	Description:
		Empty constructor for the class
	Throws:	nothing
	Class:	Board
	Returns: nothing
	Parameters:
		none
----------------------------------------------------*/

Board::Board() {
	matrix = NULL;
	width=0; height=0;
}

/*----------------------------------------------------
	Function: Board
	Description:
		Constructor that initializes the field
	Throws:	nothing
	Class:	Board
	Returns: nothing
	Parameters:
		long Width
		long Height ...coordinates of the board
----------------------------------------------------*/
Board::Board(const long Width, const long Height) {
	long memSize = sizeof(BOARD_ITEMTYPE)*Width*Height;
	matrix = NULL;
	width=0; height=0;

	matrix = (BOARD_ITEMTYPE *) malloc(memSize);
	if (matrix!=NULL) {
		//We were succesfull, go ahead
		memset(matrix,0,memSize);
		width = Width;
		height = Height;
	}
}

/*----------------------------------------------------
	Function: Board
	Description:
		Copy Constructor
	Throws:	nothing
	Class:	Board
	Returns: nothing
	Parameters:
		Board & toCopy
----------------------------------------------------*/
Board::Board(const Board & toCopy) {
	long memSize = sizeof(BOARD_ITEMTYPE)*toCopy.width*toCopy.height;
	matrix = NULL;
	width=0; height=0;

	matrix = (BOARD_ITEMTYPE *) malloc(memSize);
	if (matrix!=NULL) {
		//We were succesfull, go ahead
		memset(matrix,0,memSize);
		width = toCopy.width;
		height = toCopy.height;
	}
}

/*----------------------------------------------------
	Function: Board
	Description:
		Destructor
	Throws:	nothing
	Class:	Board
	Returns: nothing
	Parameters:
		none
----------------------------------------------------*/
Board::~Board() {
	if (matrix) {
		free(matrix);
		width=0; height=0; matrix=0;
	}
}

/*----------------------------------------------------
	Function: Initialize
	Description:
		Initializes the board
	Throws:	nothing
	Class:	Board
	Returns: true if succesfull
	Parameters:
		long newWidth
		long newHeight  ...dimensions of the board
----------------------------------------------------*/
bool Board::Initialize(const long newWidth, const long newHeight) {
	long memSize = sizeof(BOARD_ITEMTYPE)*newWidth*newHeight;

	if (matrix)
		free(matrix);
	matrix = NULL;
	width=0; height=0;

	matrix = (BOARD_ITEMTYPE *) malloc(memSize);
	if (matrix!=NULL) {
		//We were succesfull, go ahead
		memset(matrix,0,memSize);
		width = newWidth;
		height = newHeight;
		return true;
	}
	return false;
}

/*----------------------------------------------------
	Function: operator[]
	Description:
		Returns the reference to selected item
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long fieldID - the item to be selected
----------------------------------------------------*/	
BOARD_ITEMTYPE & Board::operator[] (const long fieldID) {
	if ((matrix) && (fieldID>=0) && (fieldID<=(width)*(height))) {
		return matrix[fieldID/*-1*/];
	}
	return matrix[0];
}

/*----------------------------------------------------
	Function: GetCellByID
	Description:
		Returns the reference to selected item
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long fieldID - the item to be selected
----------------------------------------------------*/	
BOARD_ITEMTYPE Board::GetCellByID(const long fieldID) {
	if ((matrix) && (fieldID>=0) && (fieldID<=(width)*(height))) {
		return matrix[fieldID/*-1*/];
	}
	return 0;	
}

/*----------------------------------------------------
	Function: GetCellByCoord
	Description:
		Returns the reference to selected item
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long coordX
		long coordY  ...the coordinates
----------------------------------------------------*/		
BOARD_ITEMTYPE Board::GetCellByCoords(const long coordX, const long coordY) {
	return GetCellByID(coordX+coordY*width/*+1*/);
}

/*----------------------------------------------------
	Function: SetCellByID
	Description:
		Sets the cell value
	Throws:	nothing
	Class:	Board
	Returns: 
		true if succesful, otherwise false
	Parameters:
		long fieldID - the item to be selected
		BOARD_ITEMTYPE & value
----------------------------------------------------*/	
bool Board::SetCellByID(const long fieldID, BOARD_ITEMTYPE value) {
	if ((matrix) && (fieldID>=0) && (fieldID<=(width)*(height))) {
		matrix[fieldID/*-1*/] = value;
		return true;
	}
	return false;	

}

/*----------------------------------------------------
	Function: SetCellByCoords
	Description:
		Sets the cell value
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long coordX
		long coordY  ...the coordinates
		BOARD_ITEMTYPE & value
----------------------------------------------------*/	
bool Board::SetCellByCoords(const long coordX, const long coordY, BOARD_ITEMTYPE value) {
	return SetCellByID(coordX +coordY*width/*+1*/, value);
}

/*----------------------------------------------------
	Function: ClearCellByID
	Description:
		Clears the cell
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long fieldID - the item to be selected
----------------------------------------------------*/	
bool Board::ClearCellByID(const long fieldID) {
	if ((matrix) && (fieldID>=0) && (fieldID<=(width)*(height))) {
		matrix[fieldID]=0;
		return true;
	}
	return false;
}

/*----------------------------------------------------
	Function: ClearCellByCoords
	Description:
		Clears the cell
	Throws:	nothing
	Class:	Board
	Returns: 
		BOARD_ITEMTYPE & pointer on the item (or NULL)
	Parameters:
		long coordX
		long coordY  ...the coordinates
----------------------------------------------------*/	
bool Board::ClearCellByCoords(const long coordX, const long coordY) {
	return ClearCellByID(coordX + coordY*width/*+1*/);
}

/*----------------------------------------------------
	Function: GetHeight
	Description:
		Returns the height of the board
	Throws:	nothing
	Class:	Board
	Returns: 
		long the height
	Parameters:
		none
----------------------------------------------------*/	
long Board::GetHeight() {
	return height;
}

/*----------------------------------------------------
	Function: GetWidth()
	Description:
		Returns the width of the board
	Throws:	nothing
	Class:	Board
	Returns: 
		long width
	Parameters:
		none
----------------------------------------------------*/	
long Board::GetWidth() {
	return width;
}

/*----------------------------------------------------
	Function: Get<MatrixPtr()
	Description:
		Returns the pointer to matrix
	Throws:	nothing
	Class:	Board
	Returns: 
		void * pointer
	Parameters:
		none
----------------------------------------------------*/	
void * Board::GetMatrixPtr() {
	return (void *) &matrix;
}

bool Board::replaceGroup(int old_val, int new_val) {
	for (int i=0; i<GetHeight()*GetWidth();i++) {
		if (GetCellByID(i) == old_val) {
			SetCellByID(i,new_val);
		}
	}
	return true;
}

bool Board::SplitBoard() {
	int red_counter=50, blue_counter=-50;
	int i,j;
	int lu,l,ld,d;  //Neighbours

// created by kulich
	for (j=0; j<GetWidth();j++)
		for (i=0; i<GetHeight();i++)
		{
			if (GetCellByCoords(j,i)!=0) {
				lu=l=ld=d=0;
				if (j!=0) {					
					l=GetCellByCoords(j-1,i);
					if (i!=0) {
						ld=GetCellByCoords(j-1,i-1);
						d=GetCellByCoords(j,i-1);
					};
					if (i!=GetHeight()-1) {
					 lu=GetCellByCoords(j-1,i+1);
					};
				}
				else {
				  if (i!=0){
				   d=GetCellByCoords(j,i-1);
				  };
				};		
			if (GetCellByCoords(j,i)>0) {
//red player			
				if (lu>0) {
				  SetCellByCoords(j,i,lu);
				  if (ld>0) replaceGroup(ld,lu);
				  else if (d>0) replaceGroup(d,lu);
				}
				else
					if (l>0) SetCellByCoords(j,i,l);
					else if (ld>0) SetCellByCoords(j,i,ld);
							 else if (d>0) SetCellByCoords(j,i,d);
							 			else {
							 				SetCellByCoords(j,i,red_counter);
							 				red_counter++;
							 			};	
			}
			else{
//blue player			
				if (lu<0) {
				  SetCellByCoords(j,i,lu);
				  if (ld<0) replaceGroup(ld,lu);
				  else if (d<0) replaceGroup(d,lu);
				}
				else
					if (l<0) SetCellByCoords(j,i,l);
					else if (ld<0) SetCellByCoords(j,i,ld);
							 else if (d<0) SetCellByCoords(j,i,d);
							 			else {
							 				SetCellByCoords(j,i,blue_counter);
							 				blue_counter--;
							 			};	
			};
		};		
  };
#ifdef DEBUG
	for (j=GetWidth()-1;j>=0;j--) {
		for (i=0; i<GetHeight();i++)
		{
			cout<<GetCellByCoords(i,j)<<" ";
		};
		cout<<endl;
	};	
#endif	
	return true;
}
