/*             _   
    /| /| /|  / \      (C) 2001  Martin Suchanek
     |/ |/  \_\_/          xsuchan@fi.muni.cz
                \   Student at Faculty of Informatics
                |       Masaryk University Brno
            \__/
*/

#ifndef __BOARD_H__
#define __BOARD_H__

#include <stdlib.h>
#include <memory.h>

//Constants:
//Type Definitions:
#define BOARD_ITEMTYPE	int

//Classes:

/*----------------------------------------------------
	Class: Board
	Description:
		Contains the current position on the board
	Variables:
		BOARD_ITEMTYPE * matrix - pointer to memory block containing the items
		long width, height - the dimensions of the items
	Functions:
		Board()	
		Board(const long width, const long height)
		Board(const Board & toCopy)
		~Board()
		bool Initialize(const long newWidth, const long newHeight)
		BOARD_ITEMTYPE & operator[] (const long fieldID)
		BOARD_ITEMTYPE & GetValueByID(const long fieldID)
		BOARD_ITEMTYPE & GetValueByCoords(const long coordX, const long coordY)
		bool SetValueByID(const long fieldID, BOARD_ITEMTYPE & value)
		bool SetValueByCoords(const long coordX, const long coordY)
		bool ClearValueByID(const long fieldID)
		bool ClearValueByCoords(const long coordX, const long coordY)
		long GetHeight()
		long GetWidth()
----------------------------------------------------*/

class Board {
private:
	BOARD_ITEMTYPE * matrix;		//The matrix that contains positions
	long width, height; //The dimensions of the board
public:
	Board();		//Default constructor
	Board(const long width, const long height);  //Constructor that creates the board
	Board(const Board & toCopy);		//Copy Costructor
	~Board();		//Default destructor

	bool Initialize(const long newWidth, const long newHeight); //Initializes the board
	BOARD_ITEMTYPE & operator[] (const long fieldID);  //Board reference as a one-row array
	BOARD_ITEMTYPE GetCellByID(const long fieldID); //Return the value stored on specified field
	BOARD_ITEMTYPE GetCellByCoords(const long coordX, const long coordY); //Returns the value stored on the field with given coordinates

	bool SetCellByID(const long fieldID, BOARD_ITEMTYPE value);	//Sets the value by id
	bool SetCellByCoords(const long coordX, const long coordY, BOARD_ITEMTYPE value);  //Sets the value by coordinates

	bool ClearCellByID(const long fieldID);  //Clears the item by id
	bool ClearCellByCoords(const long coordX, const long coordY); //Clears the item by coordinates

	long GetHeight();	//Returns the number of rows
	long GetWidth();	//Returns the number of columns

	void * GetMatrixPtr();  //Returns pointer to the matrix
	bool SplitBoard();
	bool replaceGroup(int old_val, int new_val);

};

#endif.. 