/*             _   
    /| /| /|  / \      (C) 2001  Martin Suchanek
     |/ |/  \_\_/          xsuchan@fi.muni.cz
                \   Student at Faculty of Informatics
                |       Masaryk University Brno
            \__/
*/
//This file will contain functions in openGL that manage the user's interface

#include "Interface.h"
#include "minimax.h"
#include <time.h>

extern int MakeMove();
extern bool first_turn;
  
//Few constants and global variables

GLfloat litposition[] = {0,0,3,0};
GLfloat LocalView[] = {0};

GLfloat mainwindow_width = INTERFACE_WINDOW_WIDTH;
GLfloat mainwindow_height = INTERFACE_WINDOW_HEIGHT;
GLfloat subwindow_width = INTERFACE_WINDOW_WIDTH - INTERFACE_WINDOW_INFO_WIDTH - 2*(INTERFACE_WINDOW_GAP);
GLfloat subwindow_height = INTERFACE_WINDOW_HEIGHT - 2*(INTERFACE_WINDOW_GAP);

//Information about players
int PlayerRedLastX=0, PlayerRedLastY=0, PlayerRedCount=0, PlayerRedScore=0; 
int PlayerBlueLastX=0,PlayerBlueLastY=0, PlayerBlueCount=0, PlayerBlueScore=0;
bool bFinished;

enum  
{ PERSPEKTIVA, ROVNOBEZNE_PROMITANI } mod = ROVNOBEZNE_PROMITANI ;

void redisplay_all(void);

GLuint window, world;   /* pouzijeme 2 okna */

char* volba = "zadna akce";

GLvoid *font_style = GLUT_BITMAP_TIMES_ROMAN_10;
GLboolean animace = GL_FALSE;
GLboolean MRIZ = GL_FALSE;

float deltar = 0.1; /* trivialni animace - posuv v ose x */
float zmenar = 0.05;
float deltak = 0.0;
float zmenak = 1;

void
setfont(char* name, int size)
{
    font_style = GLUT_BITMAP_HELVETICA_10;
    if (strcmp(name, "helvetica") == 0) {
	if (size == 12) 
	    font_style = GLUT_BITMAP_HELVETICA_12;
	else if (size == 18)
	    font_style = GLUT_BITMAP_HELVETICA_18;
    } else if (strcmp(name, "times roman") == 0) {
	font_style = GLUT_BITMAP_TIMES_ROMAN_10;
	if (size == 24)
	    font_style = GLUT_BITMAP_TIMES_ROMAN_24;
    } else if (strcmp(name, "8x13") == 0) {
	font_style = GLUT_BITMAP_8_BY_13;
    } else if (strcmp(name, "9x15") == 0) {
	font_style = GLUT_BITMAP_9_BY_15;
    }
}

void 
drawstr(double x, double y, char* format, ...)
{
    va_list args;
    char buffer[255], *s;
    
    va_start(args, format);
    vsprintf(buffer, format, args);
    va_end(args);
    
    glRasterPos2d(x, y);
    for (s = buffer; *s; s++)
	glutBitmapCharacter(font_style, *s);
}

/* =============================================================== */
/*   kresba matice hraciho pole                                    */
/* =============================================================== */
void drawMatrix(double x, double y, double width, double height) 
{
	int i,j;
	double stepx, stepy;
	int rows, columns;
	rows = PlayBoard.GetWidth();
	columns = PlayBoard.GetHeight();

	stepx = width/(rows); stepy = height/(columns);
	x+=stepx/2; y+=stepy/2; width-=stepx; height-=stepy;

    glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glLineWidth(1.0);
	PlayerBlueCount = 0;
	PlayerRedCount = 0;
	//Draw matrix
	for (i = 0; i<rows-1; i++) {
		for (j = 0; j<columns-1; j++) {
			glBegin(GL_LINE_STRIP);
				glColor3f(0.4,0.4,0.4);	
				//Draw one cell only
				glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*(i+1))/subwindow_height);
			glEnd();
		}
	}
	//Draw point positions
	for (i = 0; i<rows; i++) {
		for (j = 0; j<columns; j++) {
			glBegin(GL_POLYGON);
			if (PlayBoard[i*columns+j/*+1*/]>0/*==RED_PLAYER_ID*/)
			{
				PlayerRedCount++;
				glColor3f(1.0,0.4,0.2);
			} else
			if (PlayBoard[i*columns+j/*+1*/]<0/*==BLUE_PLAYER_ID*/)
			{
				PlayerBlueCount--;
				glColor3f(0.2,0.4,1.0);
			} else
				glColor3f(0.4,0.4,0.4);
				glVertex2f((x+stepx*j-INTERFACE_POINT_SIZE/2)/subwindow_width,(y+stepy*i-INTERFACE_POINT_SIZE/2)/subwindow_height);
				glVertex2f((x+stepx*j+INTERFACE_POINT_SIZE/2)/subwindow_width,(y+stepy*i-INTERFACE_POINT_SIZE/2)/subwindow_height);
				glVertex2f((x+stepx*j+INTERFACE_POINT_SIZE/2)/subwindow_width,(y+stepy*i+INTERFACE_POINT_SIZE/2)/subwindow_height);
				glVertex2f((x+stepx*j-INTERFACE_POINT_SIZE/2)/subwindow_width,(y+stepy*i+INTERFACE_POINT_SIZE/2)/subwindow_height);
			glEnd();
		}
	}
	glLineWidth(3.0);
	//Now update players
	//First red one, then blue one
	for (i = 0; i<rows-1; i++) {
		for (j = 0; j<columns-1; j++) {
			glBegin(GL_POLYGON);
				glColor3f(1.0,0.2,0.2);
				if (PlayBoard[i*columns+j/*+1*/]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i)*columns+j/*+2*/+1]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+2*/+1)]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+1*/)]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
			glEnd();
			glBegin(GL_LINE_STRIP);
				glColor3f(1.0,0.2,0.2);
				if (PlayBoard[i*columns+j/*+1*/]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i)*columns+j+1]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+2*/+1)]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+1*/)]>0/*==RED_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
			glEnd();
			glBegin(GL_POLYGON);
				glColor3f(0.3,0.4,1.0);
				if (PlayBoard[i*columns+j/*+1*/]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i)*columns+j/*+2*/+1]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+2*/+1)]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+1*/)]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
			glEnd();
			glBegin(GL_LINE_STRIP);
				glColor3f(0.3,0.4,1.0);
				if (PlayBoard[i*columns+j/*+1*/]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*j)/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i)*columns+j/*+2*/+1]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*i)/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+2*/+1)]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j+1))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
				if (PlayBoard[(i+1)*columns+(j/*+1*/)]<0/*==BLUE_PLAYER_ID*/) {
					glVertex2f((x+stepx*(j))/subwindow_width,(y+stepy*(i+1))/subwindow_height);
				}
			glEnd();
		}
	}
}

/* =============================================================== */
/*   kresba sceny do aktivniho bufferu                             */
/* =============================================================== */
void
createScene(void)
{
	GLfloat width, height;
	width = glutGet(GLUT_WINDOW_WIDTH);
	height = glutGet(GLUT_WINDOW_HEIGHT);
    drawMatrix(-width,-height,width*2,height*2);
}

void
interface_main_reshape(int width,  int height) 
{
    glViewport(0, 0, width, height);
    subwindow_width = width-INTERFACE_WINDOW_INFO_WIDTH - INTERFACE_WINDOW_GAP*2;
    subwindow_height = height-INTERFACE_WINDOW_GAP*2;
	mainwindow_width=width;
	mainwindow_height=height;
    /* se zmenou hlavniho okna se meni i vnitrni okno "world" */
    glutSetWindow(world);
    glutPositionWindow(INTERFACE_WINDOW_GAP+INTERFACE_WINDOW_INFO_WIDTH, INTERFACE_WINDOW_GAP);
    glutReshapeWindow(subwindow_width, subwindow_height);
}

void

interface_main_display(void)
{
	int y=0;
    glViewport(0, 0, mainwindow_width, mainwindow_height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, mainwindow_width, mainwindow_height, 0);
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    glClearColor(0.8, 0.8, 0.8, 0.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glColor3ub(0, 0, 0);
    setfont("helvetica", 12);
    drawstr(INTERFACE_WINDOW_GAP, INTERFACE_WINDOW_GAP-5, "Hra Oblasti");
	//Now write down information about players:
	//First Red player, then Blue player
	glColor3f(1.0,0.2,0.2);
	y = INTERFACE_WINDOW_GAP;
	setfont("helvetica",18);
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP, y, "Hrac:");
	setfont("helvetica",12);
	y+=21;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Posledni tah: [%2d,%2d]",PlayerRedLastX, PlayerRedLastY);
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Pocet obsazenych policek: %d",PlayerRedCount);
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Procentualne: %6.2f%%", (1.0*PlayerRedCount/(WIDTH*PlayBoard.GetHeight())*100));
	y+=60;
	glColor3f(0.1,0.2,1.0);
	setfont("helvetica",18);
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP, y, "Pocitac:");
	setfont("helvetica",12);
	y+=21;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Posledni tah: [%2d,%2d]",PlayerBlueLastX, PlayerBlueLastY);
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Pocet obsazenych policek: %d",abs(PlayerBlueCount));
	y+=18;
	drawstr(INTERFACE_WINDOW_GAP+20, y, "Procentualne: %6.2f%%", (1.0*abs(PlayerBlueCount)/(WIDTH*PlayBoard.GetHeight())*100));

	y+=60;
	if (abs(PlayerBlueCount) + PlayerRedCount == WIDTH*HEIGHT) {
		if (!bFinished) {
			bFinished = true;
			if (abs(PlayerBlueCount) > PlayerRedCount) PlayerBlueScore++;
			if (abs(PlayerBlueCount) < PlayerRedCount) PlayerRedScore++;
			if ((PlayerBlueScore>=10) || (PlayerRedScore>=10)) {
				PlayerBlueScore = 0; PlayerRedScore = 0;
			}
		}
	}
	setfont("helvetica",18);
	glColor3f(0.0,0.0,0.0);
	drawstr(INTERFACE_WINDOW_GAP, y, "Skore: ");
	glColor3f(1.0,0.2,0.2);
	drawstr(INTERFACE_WINDOW_GAP+60, y, "%d",PlayerRedScore);
	glColor3f(0.0,0.0,0.0);
	drawstr(INTERFACE_WINDOW_GAP+74, y, ":");
	glColor3f(0.1,0.2,1.0);
	drawstr(INTERFACE_WINDOW_GAP+80, y, "%d",PlayerBlueScore);
	y+=20;
	if (bFinished) {
		if (abs(PlayerBlueCount)>PlayerRedCount) {
			glColor3f(0.1,0.2,1.0);
			drawstr(INTERFACE_WINDOW_GAP,y,"Vyhral pocitac");
		} else {
			if (PlayerRedCount>abs(PlayerBlueCount)) {
				glColor3f(1.0,0.2,0.2);
				drawstr(INTERFACE_WINDOW_GAP,y,"Vyhral hrac");
			} else {
				glColor3f(0.0,0.0,0.0);
				drawstr(INTERFACE_WINDOW_GAP,y,"Remiza");
			}
		}
	}
    glutSwapBuffers();


}


void
interface_main_keyboard(unsigned char key, int x, int y)
{
    switch (key) {
    case 'p':
        mod = PERSPEKTIVA;
        glutSetWindow(world); /* ke zmene muze dojit v okne "world" */
        glutPostRedisplay();
	break;
    case 'r':
        mod = ROVNOBEZNE_PROMITANI;
	break;
        glutSetWindow(world); /* ke zmene muze dojit v okne "world" */
        glutPostRedisplay();
    case 27:
	exit(0);
    }
}

void interface_main_menu(int value)
{

    switch (value) {
    case '1':	
    {	
		bFinished = false;
		for (int i=0; i<PlayBoard.GetHeight()*PlayBoard.GetWidth();i++)
			PlayBoard[i]=0;

		PlayerRedCount=0;
		PlayerBlueCount=0;
		PlayerRedLastX=0;
		PlayerBlueLastX=0;
		PlayerRedLastY=0;
		PlayerBlueLastY=0;
		first_turn=true;
	    glutPostRedisplay();
		
/*	prikaz 1;  */
			break;
		};
    case '2':
/*  prikaz 2; */
		glutDestroyWindow(world);
		glutDestroyWindow(window);
		exit(0);
		break;
//    case '3':
/*  prikaz 3; */
/*		PlayBoard.SplitBoard();
		for (int i=0; i<PlayBoard.GetHeight(); i++) {
			for (int j=0; j<PlayBoard.GetWidth(); j++);
				cout << PlayBoard.GetCellByCoords(i,j) << " ";
			cout << endl;
		}

		break;
*/
  }

}

void setProjection(void)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
//	switch (mod) {
//	case (PERSPEKTIVA):
		glFrustum(-1.0,1.0,-1.0,1.0,1.0,20.0);
//		break;
//	case (ROVNOBEZNE_PROMITANI):
//        glOrtho(-1.0,1.0,-1.0,1.0,-20.0,20.0);
//		break;
//	}
glMatrixMode(GL_MODELVIEW);
}


void
interface_world_reshape(int width, int height)
{
    glViewport(0, 0, width, height);
    setProjection();
}


void 
interface_world_idle(void)
{
    if (animace) {
      glutPostRedisplay();
    }
}

void
interface_world_display(void)
{
    /* zobrazeni sceny v okne 'world' */
    glutSetWindow(world);
    /* uprav velikost - pokud je treba ... */
    interface_world_reshape(subwindow_width, subwindow_height);
    /* smaz predchozi obsah  */
    glClearColor(0.0,0.0,0.0,0.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    /* namaluj pozadovanou kresbu */
    createScene(); 
    /* konec zmen, pozadej o prekresleni */
    glutPostRedisplay();
    /* v dalsi etape pouzij druhy buffer */
    glutSwapBuffers(); 
}

void
interface_world_menu(int value)
{
    glutSetWindow(world);
    switch (value) {
    case 'a':
	/* akce_a(); */
		volba="akce a"; 
		animace=GL_FALSE;
                glutPostRedisplay();
	break;
    case 'b':
	/* akce_b(); */
		volba="akce b"; 
		animace=GL_TRUE;
		MRIZ=GL_TRUE;
                glutPostRedisplay();
	break;
    case 'c':
	animace = GL_TRUE;
        volba="akce c";
		MRIZ=GL_FALSE;
	/*        glutPostRedisplay(); ... nemusi zde byt, zaridi "world_idle"  */
    }
}

void interface_world_mouse(int button, int state, int x, int y)
{
	int i,j;
	float stepx, stepy;
	int rows, columns;
	rows = PlayBoard.GetWidth();
	columns = PlayBoard.GetHeight();

	if (state == 1) return; //Don't do it twice
	stepx = subwindow_width/(rows); stepy = subwindow_height/(columns);
	//Now, try to find selected field

	i = (y / stepy);
	j = (x / stepx);

	i = rows - i;
	i--;
	
 	printf("??: %d x %d\n",j,i);
	if ((i<0) || (j<0)) return;
	if (PlayBoard.GetCellByCoords(j,i)==0) {
		//Set new value for this item
		//Use constant from minimax.h
		if (GLUT_LEFT_BUTTON == button) {
			MinMax minimax;
			Board returned_board(WIDTH,HEIGHT);
			if (minimax.DetectCycles(1,j+i*WIDTH, returned_board)) {
				PlayBoard[j+i*WIDTH]=1;
				returned_board[j+i*WIDTH]=10;
				List<PointToReturn> * list_to_return;
				int blue_count=0;
				list_to_return=minimax.CloseCyclesOnBoard(1,returned_board,blue_count);
				for (int i=0; i<list_to_return->GetCard(); i++) {
					PlayerRedCount++;
					PlayBoard[(*list_to_return)[i].GetPosition()]=1;
				};

			delete(list_to_return);	
			}
			else PlayBoard[j+i*WIDTH]=1;

			printf("red:  %d x %d\n",j,i);
			PlayerRedLastX = j/*+1*/; PlayerRedLastY = i/*+1*/; PlayerRedCount++;
			glutSetWindow(window);
   			glutPostRedisplay();

// Now do the movement
			clock_t start_time=clock();
			j = MakeMove();
			cout<<"Souper odpovedel za "<<clock()-start_time
				<<" milisekund"<<endl;
			i = j / PlayBoard.GetWidth();
			j = j % PlayBoard.GetWidth();
			PlayerBlueLastX = j/*+1*/; PlayerBlueLastY = i/*+1*/; PlayerBlueCount++;
		}
/*		if (GLUT_RIGHT_BUTTON == button) {
			MinMax minimax;
			Board returned_board(WIDTH,HEIGHT);
			if (minimax.DetectCycles(0,j+i*WIDTH, returned_board)) {
				PlayBoard[j+i*WIDTH]=-1;
				returned_board[j+i*WIDTH]=10;
				List<PointToReturn> * list_to_return;
				int blue_count=0;
				list_to_return=minimax.CloseCyclesOnBoard(0,returned_board,blue_count);

				for (int i=0; i<list_to_return->GetCard(); i++) {
					PlayerBlueCount--;
					PlayBoard[(*list_to_return)[i].GetPosition()]=-1;
				};

				delete(list_to_return);	

			}
			else PlayBoard[j+i*WIDTH]=-1;

			printf("blue: %d x %d\n",j,i);
			PlayerBlueLastX = j/*+1*/;
//			PlayerBlueLastY = i/*+1*/;
//			PlayerBlueCount--;
/*			glutSetWindow(window);
		    glutPostRedisplay();
		}
*/
	}
	else {
		long num=PlayBoard.GetCellByCoords(j,i);
		if ((GLUT_LEFT_BUTTON == button)&&(num>0)) {
			num++;
			PlayBoard.SetCellByCoords(j,i,num/*RED_PLAYER_ID*/);
		}
		if ((GLUT_RIGHT_BUTTON == button)&&(num<0)) {
			num--;
		  PlayBoard.SetCellByCoords(j,i,num/*BLUE_PLAYER_ID*/);
		};
	};

}
int InitWindow(int * argc, char ** argv)  //The same arguments as in main function
{

    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE); 
    glutInitWindowSize(mainwindow_width, mainwindow_height);
    glutInitWindowPosition(50, 50);
    // glutInit(argc, argv);


	/*  hlavni okno se jmenuje 'window' */
    window = glutCreateWindow("Oblasti");
    glutReshapeFunc(interface_main_reshape);
    glutDisplayFunc(interface_main_display);
    glutKeyboardFunc(interface_main_keyboard);
    glutCreateMenu(interface_main_menu);
    glutAddMenuEntry("Hlavni menu", 0);
    glutAddMenuEntry("", 0);		   /* prazdny radek v menu */
    glutAddMenuEntry("Nova hra", '1');  /* vrati znak '1' */
//	glutAddMenuEntry("Seskup prvky", '3');  /* vrati znak '1' */
    glutAddMenuEntry("Konec", '2');  /* vrati znak '2' */
    glutAttachMenu(GLUT_RIGHT_BUTTON); 
                        /* menu se zobrazi po stisku praveho tlacitka mysi */
    glutAttachMenu(GLUT_LEFT_BUTTON); 
                        /* menu se zobrazi po stisku leveho tlacitka mysi  */

    /* okno 'world'  */
    world = glutCreateSubWindow(window, INTERFACE_WINDOW_GAP+INTERFACE_WINDOW_INFO_WIDTH, INTERFACE_WINDOW_GAP, subwindow_width, subwindow_height);
    glutReshapeFunc(interface_world_reshape);
    glutDisplayFunc(interface_world_display);
    glutKeyboardFunc(interface_main_keyboard);
    glutCreateMenu(interface_world_menu);
	  glutMouseFunc(interface_world_mouse);
    glutAddMenuEntry("Menu okna 'world'", 0);
    glutAddMenuEntry("", 0);		   /* prazdny radek v menu */
    glutAddMenuEntry("volba - a", 'a');
    glutAddMenuEntry("volba - b", 'b');
    glutAddMenuEntry("volba - c", 'c');
    glutAttachMenu(GLUT_MIDDLE_BUTTON); 
                  /* menu se zobrazi po stisku praveho tlacitka mysi */

    glutIdleFunc (interface_world_idle);  /* world_idle muze byt pouzita pro animaci */

	glEnable(GL_AUTO_NORMAL);
	glEnable(GL_NORMALIZE);
	glShadeModel(GL_SMOOTH);

    glEnable(GL_DEPTH_TEST);
    glEnable(GL_CULL_FACE);
    glutMainLoop();

    return 0;
}